      Program SMC_HDRS

*  Reads a list of smc file names, and for each entry
*  it opens the data file and reads header information.  This 
*  information is then written to a file.  This eliminates the need 
*  to manually list the contents of each file to check on 
*  components, station coordinates, and so on.

* To indicate end of list, insert a line with 'stop' starting
* in any column

* Here is a sample input file:

*Control file for program SMC_Hdrs
*Summary file:
*  as.sum
*Names of SMC files:
*ALHAMBRA.AS1
*ALHAMBRA.AS2
*ALHAMBRA.AS3
*ANACAPA.AS1
*ANACAPA.AS2
*ANACAPA.AS3
* stop                

*  Dates:  12/29/97 - Written by D. Boore, based on CDROMLST
*          01/15/99 - continued debugging
*          01/16/99 - extensive rewrite, output now includes all of the
*                     header information for each file
*          10/25/00 - Removed restriction that "stop" must be in column 2
*          04/30/02 - Suggest default name of control file

      character f_ctl*40, f_sum*40, f_smc*40, head_c*80, dash_c(80)*1

      real real_head(50)
      integer int_head(50)

      logical f_ctl_exist

      data dash_c/80*'-'/
* Get the name of the file with file names:

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = smc_hdrs.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smc_hdrs.ctl'
        inquire(file=f_ctl, exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl, status='unknown')


*  Get the name of the summary file:

      call skip(nu_ctl, 2)      

      f_sum = ' '
      read(nu_ctl, '(a)' ) f_sum

      call get_lun(nu_sum)
      open (unit=nu_sum, file=f_sum, status='unknown')
      write(nu_sum,'(a)') 
     :    ' ******* Output of program SMC_HDRS *********'

* Skip header:

      call skip(nu_ctl, 1)      

* Open each file
      
1000  continue

      f_smc = ' '
      read(nu_ctl, '(a)', end = 999) f_smc
      call upstr(f_smc)
      call trim_c(f_smc, nchar)

      if (f_smc(1:4) .eq. 'STOP') then
        close(unit = nu_ctl)
        stop
      end if

      call get_lun(nu_smc)
      open (unit=nu_smc, file=f_smc, status='unknown')

      call trim_c(f_smc,nc_f_smc)      

      write(nu_sum,*)
      write(nu_sum,'(3a)') 
     :  ' Headers for smc file: ', f_smc(1:nc_f_smc), ':'
      write(nu_sum,*)

* Read and write the text headers:

      do i = 1, 11
        head_c = ' '
        read(nu_smc,'(a)') head_c
        write(nu_sum,'(a)') head_c
      end do


* Read and write the integer headers:
      
      read(nu_smc,'(8i10)') (int_head(i), i = 1, 48)
      write(nu_sum,'(8i10)') (int_head(i), i = 1, 48)
     
* Read and write the real headers:

      read(nu_smc,'(5e15.7)') (real_head(i), i = 1, 50)
      write(nu_sum,'(5e15.7)') (real_head(i), i = 1, 50)

* Read and write the comment lines:
     
      n_comment = int_head(16)

      do i = 1, n_comment
        head_c = ' '
        read(nu_smc,'(a)') head_c
        write(nu_sum,'(a)') head_c
      end do

* Finished with this file; loop back for a new file:

      write(nu_sum,*)
      write(nu_sum,'(80a1)')  (dash_c(i), i = 1, 80)


      close(unit=nu_smc)

      go to 1000        

* Get here only if END-OF-FILE is encountered
999   continue

      close(unit=nu_sum)

      stop
      end

      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\skip.for'
      include '\forprogs\get_lun.for'
   