* ---------------------------------------------------------------- ColSpect2GM_AR
      Program ColSpect2GM_AR

* Reads a pair of filenames from a control file, for which each file is a column file
* containing a column of frequency and another of soectra (they could be Fourier or response
* spectra).  Compute the as-recorded geometric mean, the sqrt sum of squares of the
* spectra (note: the meaning of this is questionable for response spectra), and writes to
* a file.

* Note: This version assumes that the number of frequencies is in the line before the 
* line containing the column headers (as is true for files made using smc2fs2).
*
* "AR" = "As Recorded"
 
* Structure of control file:
* Line 1 follows:
*!Control file for program ColSpect2GM_AR
*!Summary file name:
* colspect2gm_ar.sum
*!Line containing the number of frequencies
*  3
*! Extension for output file (append to end of first file name)
* .gm_ar.asc
*! list of file names (assumes pairs of files) 
*    file1_comp1.fs.col
*    file1_comp2.fs.col
*    file2_comp1.fs.col
*    file2_comp2.fs.col
*   StoP
 
 
* Dates: 03/25/08 - Written by D. Boore, based on smc2gm_ar.for.
*        03/28/08 - Added line with number of frequencies to control file.
    
      character char_head(11,2)*80, comments(200,2)*80
      character cmnts2skip(20)*80, path_in*40, path_out*40      
       
      integer idate_time_values(8)
      
      real time_sec_start, time_sec_stop

      integer int_head(48,2), nc_f_in(2),  nc_f_out, nc_rc(2)
      integer nfreq(2)

      real real_head(50,2) 
      real freq(:,:), spect(:,:) 
      allocatable :: freq, spect  
       
      character buf_c*80, buf4_c*4, f_ctl*80, f_sum*80,
     :          buf_out*79
      character  f_in(2)*80, f_out*100, f_ext*80, f_penalty*100,
     :           c_az*7, rc(2)*12
      character per4az_c*5, colhead1(10)*12, colhead2(10)*12, frmt_c*80
      character use_nga_per4gm*10
      logical f_ctl_exist
      logical rmv_trnd
      
      pi = 4.0*atan(1.0)
      twopi = 2.0*pi
      d2r = pi/180.0
      nper_dimen = 300
      nazm_dimen = 400

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = colspect2gm_ar.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'colspect2gm_ar.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip, nc_comments)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call get_lun(nu_sum)
      call trim_c(f_sum,nc_f_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')    
      write(nu_sum,'(a)') ' Summary file for program ColSpect2GM_AR'
      write(nu_sum,'(2a)') '   Using control file: ', f_ctl
      do i = 1, nc_comments
        write(nu_sum,'(a)') cmnts2skip(i)
      end do
      write(nu_sum,'(a)') f_sum

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) line_with_nfreq
       
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      f_ext = ' '
      read(nu_ctl,'(a)') f_ext
      call trim_c(f_ext,nc_f_ext)
      write(nu_sum,'(a)') f_ext(1:nc_f_ext)
         
 

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      write(nu_sum, '(a)') ' Just before 1000 continue '

1000  continue ! Loop over files

        write(nu_sum,*)

 
* Read filenames and fill frequency,spectra arrays:
        do j = 1, 2
          buf_c = ' '
          read(nu_ctl,'(a)') buf_c
          call trim_c(buf_c,nc_buf)
          buf4_c = ' '
          buf4_c = buf_c(1:4)
          call upstr(buf4_c)

          if (buf4_c(1:4) .eq. 'STOP') then
            write(*, '(a)') 
     :           ' buf_c = '//buf_c(1:nc_buf)//'; QUITTING'
            close(unit=nu_sum)
            close(nu_ctl)
            stop
          end if

          call date_and_time(values=idate_time_values)
          time_sec_start = float(idate_time_values(5)*60*60+
     :                           idate_time_values(6)*60   +
     :                           idate_time_values(7))     +
     :                     float(idate_time_values(8))/1000.0
     
          f_in(j) = ' '
          f_in(j) = buf_c(1:nc_buf)
          call trim_c(f_in(j),nc_f_in(j))
          
          call get_lun(nu_spect)
          open(nu_spect,file=f_in(j)(1:nc_f_in(j)),status='unknown')
          
          call skip(nu_spect,line_with_nfreq-1)
          read(nu_spect,*) nfreq(j)
        
          close(nu_spect)
        end do
        
        max_dimen = max0(nfreq(1), nfreq(2))
        allocate( freq(max_dimen,2), spect(max_dimen,2) )
          
        do j = 1, 2
        
          call get_lun(nu_spect)
          open(nu_spect,file=f_in(j)(1:nc_f_in(j)),status='unknown')
          
          call skip(nu_spect,line_with_nfreq+1)
          
          do i = 1, nfreq(j)
            read(nu_spect,*) freq(i,j), spect(i,j)
          end do
          
          close(nu_spect)
          
          write(nu_sum,'(a,i1,a)') 
     :                 ' f_in(',j,') = '//f_in(j)(1:nc_f_in(j))
          print *,' f_in(',j,') = '//f_in(j)(1:nc_f_in(j))
          
        end do

* Check for equality of frequencies:

        if (nfreq(1) .ne. nfreq(2)) then
           print *,' ERROR: nfreq not equal; move to next pair'
           write(nu_sum,'(a)') 
     :         ' ERROR: nfreq not equal; move to next pair'
           write(nu_sum,'(a,1x,i4, 1x, i4)') 
     :         ' nfreq(1,2) = ', nfreq(1), nfreq(2)
           go to 1000
        end if
        
        do i = 1, nfreq(1)
          if (freq(i,1) .ne. freq(i,2)) then
             print *,' ERROR: freq not equal; move to next pair'
             write(nu_sum,'(a)') 
     :         ' ERROR: freq not equal; move to next pair'
             write(nu_sum,'(a,1x,i4, 1x, i4)') 
     :         ' freq(i,1,2) = ', freq(i,1), freq(i,2)
             go to 1000
          end if
        end do
        
* Passed tests, now combine spectra and write to a file        
        
        f_out = ' '
        f_out = f_in(1)(1:nc_f_in(1))//f_ext(1:nc_f_ext)
        call trim_c(f_out,nc_f_out)
        call get_lun(nu_out)
        open(nu_out,file=f_out(1:nc_f_out),status='unknown')
        write(nu_sum,'(a)') ' Output file = '//f_out(1:nc_f_out)
        
        write(nu_out,'(a)') ' input file 1: '//f_in(1)(1:nc_f_in(1))
        write(nu_out,'(a)') ' input file 2: '//f_in(2)(1:nc_f_in(2))
        write(nu_out,'(5x,a, 
     :                 3x,a, 3x,a,
     :                 1x,a, 2x,a,
     :                 2x,a 
     :                  )') 
     :    'freq', 
     :    'spct1_ar', 'spct2_ar', 
     :    'spctlarger_ar', 'spctgm_ar',
     :    'srss12_ar' 
      
      
        do i  = 1, nfreq(1)
        
          spect_larger = spect(i,1)
          if (spect(i,2) .gt. spect_larger) then
            spect_larger = spect(i,2)
          end if
          
          spect_gm = sqrt(spect(i,1)*spect(i,2))
          spect_srss = sqrt(spect(i,1)**2+spect(i,2)**2)
            
          write(nu_out,'(1x,f8.4, 
     :               1p,
     :                   1x,e10.3, 1x,e10.3,
     :                   4x,e10.3, 1x,e10.3,
     :                   1x,e10.3, 1x,e10.3, 
     :                   4x,e10.3, 1x,e10.3
     :                                                      )') 
     :      freq(i,1), 
     :      spect(i,1), spect(i,2), 
     :      spect_larger, spect_gm,
     :      spect_srss

        end do

        close (nu_out)
        
        deallocate( freq, spect )
          
       
        call date_and_time(values=idate_time_values)
        time_sec_stop = float(idate_time_values(5)*60*60+
     :                           idate_time_values(6)*60   +
     :                           idate_time_values(7))     +
     :                    float(idate_time_values(8))/1000.0
     
       write(nu_sum,'(a,1x,f7.3)') ' calc time = ',
     :                     time_sec_stop - time_sec_start
     
      go to 1000    ! loop back for another pair of components

      end
* ---------------------------------------------------------------- ColSpect2GM_AR

      include '\forprogs\acc2vd.for'     
      include '\forprogs\Trapezoidal_Integration.for'
      include '\forprogs\rmvtrend.for'     
      include '\forprogs\rotate.for'     
      include '\forprogs\get_lun.for'     
      include '\forprogs\skip.for'     
      include '\forprogs\smcread.for'     
      include '\forprogs\mnmaxidx.for'     
      include '\forprogs\abs_mnmaxidx.for'     
      include '\forprogs\trim_c.for'     
      include '\forprogs\upstr.for'     
      include '\forprogs\smc_npts.for'     
      include '\forprogs\distaz.for'     
      include '\forprogs\rc_subs.for'     
      include '\forprogs\skipcmnt.for'     
      include '\forprogs\rdrvaa.for'
      include '\forprogs\moment.for'
      include '\forprogs\select.for'
  

*      subroutine RCF (KPARM,CSTR_IN,NCSTR_IN,F,ibgn,iend, ISTAT)

 
*Write a file with az1, daz, (period, gmean, gmean-%, gmean_50%, gmean+%), repeated for a set of periods specficied in input  


      
