* ------------------ BEGIN FS2_Avg -----------------------------
      Program FS2_Avg

* Read a control file with info for 1 to 72 Fourier amplitude
* spectra in smc format.  Read
* the smc file and reformat to produce a column file, including
* the average of the spectra.  

* It is assumed that the frequency points are the same for all spectra.

* Here as a sample input file:

*!Control file for FS2_Avg      ! first line
*!Name of output file:
* fs2_avg.col
*! List of files for which to compute the average Fourier transform.  It is 
*! assumed that these files in smc format.  
* file1.fsc
* file2.fsc
* file3.fsc
*stop

* Dates: 08/04/02 - Written by D. Boore (patterned after smc2asc.for)

      parameter(ny_max = 72)
      real y(:,:), avg(:), avg_sq(:), avg_log(:), navg_log(:)
      allocatable :: y, avg, avg_sq, avg_log, navg_log

      real real_head(50)
      integer int_head(48), nc_f_y
      character char_head(11)*80, comments(500)*80, cmnt_ctl(100)*80
      character f_ctl*60, f_out*60, f_y*60, buf*100,
     :  colhead(ny_max)*14, 
     :  comp_c*1, frmt_c*40, sep_c*1
      logical f_ctl_exist, f_y_exist

* Get name of input file:
      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=FS2_Avg.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'fs2_avg.ctl'
        inquire(file=f_ctl, exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

* Open file 
      call get_lun(nu_ctl)      
      open(unit=nu_ctl, file=f_ctl, status='unknown')

* Skip header lines:
      call skipcmnt(nu_ctl,cmnt_ctl,ncmnt_ctl)

* Read output file name:
      f_out = ' '
      read(nu_ctl, '(a)') f_out
      call trim_c(f_out,nc_f_out)

*  Skip text lines:
      call skipcmnt(nu_ctl,cmnt_ctl,ncmnt_ctl)

* Loop over spectra files:
      ny = 0
1000  continue

* Read file name:
        buf = ' '
        read(nu_ctl, '(a)') buf
        call upstr(buf)
        call trim_c(buf,nc_buf)

* Check for "STOP":
        if (buf(1:4) .eq. 'STOP') go to 2000

* Check for ny > max number
        if (ny + 1 .gt. ny_max) then
           write(*,*)
           write(*, '(a   )') 
     :    ' ny+1 > ny_max; ignoring remaining y'
           write(*,*)
           go to 2000
        end if

* Check for existence of file and skip if it does not exist.
        f_y = ' '
        f_y = buf(1:nc_buf)
        call trim_c(f_y, nc_f_y)
        inquire(file=f_y, exist=f_y_exist)
        if (.not. f_y_exist) then
          write(*,'(3a)') ' ****** FILE ', f_y, 
     :          ' DOES NOT EXIST; Skipping!! ******* '
          go to 1000
        end if

* Get various information from headers:
        call ReadHdrs(f_y(1:nc_f_y), 
     :                char_head, int_head, real_head, comments)

* If not the first file, check to make sure frequency points are the same
* and skip if not:

        if (ny+1 .eq. 1) then
          npy1 = int_head(45)
          flow1 = real_head(42)
          df_fas1 = real_head(44)
          allocate( y(npy1,ny_max), 
     :              avg(npy1), avg_sq(npy1), avg_log(npy1), 
     :              navg_log(npy1))
          do i = 1, npy1
            avg(i) = 0.0
            avg_sq(i) = 0.0
            avg_log(i) = 0.0
            navg_log(i) = 0
          end do
        else
          if (int_head(45) .ne. npy1 .or.
     :        real_head(42) .ne. flow1 .or.
     :        real_head(44) .ne. df_fas1) then
            write(*,'(a)') 
     :        ' File '//f_y(1:nc_f_y)//
     :        ' has different frequency points than file1; skipping.'
            write(*,'(a)') ' npy1, flow1, df_fas1 = '
            write(*,*)  npy1, flow1, df_fas1
            write(*,'(a)') ' npy(j), flow(j), df_fas(j) = '
            write(*,*) int_head(45), real_head(42), real_head(44)            
            go to 1000
          else if(char_head(1)(1:1) .ne. '6') then  ! not Fourier spectra
            write(*,'(a)') 
     :        ' File '//f_y(1:nc_f_y)//
     :        ' does not contain Fourier spectra; skipping.'
            go to 1000
          end if
        end if

* Passed tests, proceed:
        ny = ny + 1
        write(*,'(2a)') '  f_y = ', f_y(1:nc_f_y)

        inull = int_head(1)
        ivert = int_head(13)
        if (ivert .eq. inull .or. ivert .eq. 90) then
          comp_c = 'H'
        else
          comp_c = 'V'
        end if

* Set up the column headings:

* Set separator to blank:
        sep_c = '_'

        colhead(ny) = ' '
        colhead(ny)(1:14) = f_y(1:nc_f_y)//
     :                 sep_c//comp_c//'............'
        write(*,'(2a)') ' colhead = ', colhead(ny)

* Now read the data from the files:
     
        call FS2Read(f_y(1:nc_f_y), 
     :                   y(1,ny), 
     :                   char_head, int_head, real_head, comments)

        do i = 1, npy1
          avg(i) = avg(i) + y(i,ny)
          avg_sq(i) = avg_sq(i) + y(i,ny)*y(i,ny)
          if (y(i,ny) .ne. 0.0) then
            avg_log(i) = avg_log(i) + alog10(y(i,ny))
            navg_log(i) = navg_log(i) + 1
          end if          
        end do

        go to 1000  ! loop back for another file

2000  continue

      close(unit=nu_ctl)

* Write output:

* Open output file:

      write(*,*)
      write(*,'(a)') ' Writing output file: '//f_out(1:nc_f_out)

      call get_lun(nu_out)      
      open(unit=nu_out, file=f_out, status='unknown')

      frmt_c = ' '
      frmt_c = '(11x,a,xx(1x,a14),12x,a,9x,a,8x,a)'
      write(frmt_c(8:9), '(i2)') ny

      
      write(nu_out, frmt_c) 
     :  'freq', (colhead(j), j = 1, ny), 'avg', 'avg_sq', 'avg_log'
      
      frmt_c = ' '
      frmt_c = '(1p,xx(1x,e14.5))'
      write(frmt_c(5:6), '(i2.2)') ny+4

      write(*,'(a)') ' frmt_c = '//frmt_c
      write(*,'(a,1x,i5)') ' npy1 = ', npy1
      write(*,'(a,1x,i5)') ' ny = ', ny


      do i = 1, npy1

        write(nu_out, frmt_c) 
     :    float(i-1)*df_fas1+flow1,
     :    (y(i,j), j = 1, ny), 
     :    avg(i)/float(ny), sqrt(avg_sq(i)/float(ny)), 
     :    10.0**( avg_log(i)/float(navg_log(i)) )

      end do

      deallocate(y, avg, avg_sq, avg_log, navg_log)

      close(unit=nu_out)

      stop
      end
* ------------------ END FS2_Avg -----------------------------

      include '\forprogs\fs2Read.for'
      include '\forprogs\readhdrs.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\imnmax.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\upstr.for'


