! ------------------ BEGIN SMC2COL -----------------------------
      Program SMC2COL

! Convert smc files to column files (one output file per inout file, unlike smc2asc).

! Here as a sample input file:

!!Control file for SMC2COL      ! first line
!!
!! Revision of program involving a change in the control file on this date:
!   07/03/15
!!
!! Note on use of "!" to denote a comment line.  
!!
!! Processing parameters ndecim, skip, tlength, xfactr, tshift (+ to right)
!! followed by the list for which these parameters will be used.  The column
!! labels will be made up of the file name (and no path)
!! with "V" or "H" appended on the right to indicate component.  The line
!! with the processing parameters must have the string "pp:" somewhere before
!! the parameters
!! Write headers and comments (Y/N)?
! Y
!!Processing parameters ndecim, tskip, tlength, xfactr, tshift (+ to right)
!   1  0.0 999.0 +1 0.0
!!List of files
!\psv\taiwan99\t0781pea.smc
! file2.smc
! stop

! Dates: 07/03/15 - Written by D. Boore, patterned after smc2asc

      implicit none
      
      real y(:), sps, tshift, 
     :     tskip_in, tlength_in,
     :     tskip, tlength
      real spsb4, tlengthb4
      allocatable :: y

      integer nc_f_ctl, nu_ctl,	nc_ctl_cmmnts, 
     :  nc_date_ctl_in, nc_date_ctl, nctl_cmmnts,
     :  nc_buf, npts, nt, j, nu_col, i, nc_date_ctl_correct  
 
      integer npy, npyb4, 
     :        ndecim 
      real real_head(50) 
      real xfactr
      integer int_head(48), nc_f_smc, nc_f_col
      
      integer iostatus, nc_write_headers

      character date_ctl_correct*8, date_ctl_in*30, write_headers*10

      character char_head(11)*80, comments(500)*80, ctl_cmmnts(100)*80
      character f_ctl*200, f_col*200, f_smc*200, buf*200, buf4*4
      logical f_ctl_exist, f_smc_exist

! DEBUG
!      call get_lun(nu_dbg)
!      open(unit=nu_dbg, file='debug.out', status='unknown')
! DEBUG



! Get name of input file:
      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=SMC2COL.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smc2col.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

! Open file 
      call get_lun(nu_ctl)      
      open(unit=nu_ctl, file=f_ctl(1:nc_f_ctl), status='unknown')

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '07/04/15'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) /= 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        write(*,'(a)') 
     :     ' The date for the control file you used is '
     :       //date_ctl_in(1:nc_date_ctl_in)
        write(*,'(a)') 
     :     ' The correct date is '
     :       //date_ctl_correct(1:nc_date_ctl_correct)
        close(nu_ctl)
        stop
      end if
      
! Write headers and comments?:
      call skipcmnt(nu_ctl,ctl_cmmnts,nctl_cmmnts)
      write_headers = ' '
      read(nu_ctl,'(a)') write_headers
      call trim_c(write_headers, nc_write_headers)
      call upstr(write_headers)
      
! Get processing parameters:
      call skipcmnt(nu_ctl,ctl_cmmnts,nctl_cmmnts)
      read(nu_ctl,*) 
     :  ndecim, tskip_in, tlength_in, xfactr, tshift

! Loop over smc files:
      call skipcmnt(nu_ctl,ctl_cmmnts,nctl_cmmnts)

      loop_over_files: DO
        buf = ' '
        read(nu_ctl,'(a)',IOSTAT=iostatus) buf
        if (iostatus /= 0) EXIT
        call trim_c(buf,nc_buf)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        if(nc_buf .eq. 0 .or. buf(1:1) .eq. '!') CYCLE
        if(buf4(1:4) .eq. 'STOP') EXIT
        f_smc = ' '
        f_smc = buf(1:nc_buf)
        call trim_c(f_smc, nc_f_smc)
        inquire(file=f_smc(1:nc_f_smc), exist=f_smc_exist)
        if (.not. f_smc_exist) then
          write(*,'(a)') ' ****** FILE '//f_smc(1:nc_f_smc)// 
     :          ' DOES NOT EXIST; CYCLE TO NEXT FILE!!! ******* '
          CYCLE
        end if
        
! DEBUG
!        write(nu_dbg,'(a)') '  processing line: '
!        write(nu_dbg,'(2x,a)') buf(1:nc_buf)
!         write(nu_dbg,'(a)') 
!     :  ' ndecim_in, tskip_in, tlength_in, xfactr_in, tshift_in:'
!        write(nu_dbg,*) 
!     :    ndecim_in, tskip_in, tlength_in, xfactr_in, tshift_in
! DEBUG

        
        call smc_npts(f_smc(1:nc_f_smc), npts, sps)

! Get various information from headers:
!        call ReadHdrs(f_smc(1:nc_f_smc), 
!     :                char_head, int_head, real_head, comments)
  
        allocate( y(npts))

! Now read the data from the files:

        tskip = tskip_in
        tlength = tlength_in
        call SMCRead(f_smc(1:nc_f_smc), tskip, tlength,
     :               y,  
     :               char_head, int_head, real_head, comments)
            npy = int_head(17)
            sps = real_head(2)

        write(*,'(a)') '  Processing file '//f_smc(1:nc_f_smc)//
     :      ' npts, npy ='
        write(*,*) npts, npy

! DEBUG
!          write(*,*) ' Inside smc2asc. f_y  = ', 
!     :                 f_y(i)(1:nc_f_y(i)) 
!          write(*,*) ' Inside smc2asc. tskip(i), tlength(i) = ', 
!     :                 tskip(i), tlength(i)
!          write(*,*) 
!     :  ' Inside smc2asc. i, ny, a_time_series(i), npy(i), sps(i) = ', 
!     :                    i, ny, a_time_series(i), npy(i), sps(i)
!          write(*,*) ' Inside smc2asc. f_y, y(1) = ', 
!     :                 f_y(i)(1:nc_f_y(i)), y(1,i)
! DEBUG

! Decimate:
         npyb4  = npy 
         spsb4  = sps 
         tlengthb4  = tlength 
         nt = 0
         do j = 1, npy, ndecim 
           nt = nt + 1
           y(nt) = xfactr * y(j)
         end do
         npy = nt
         sps = sps/float(ndecim)
         tlength = float(npy-1)/sps
         int_head(17) = npy
         real_head(2) = sps
 
! Write output:

! Open output file:

        f_col = ' '
        f_col = f_smc(1:nc_f_smc)//'.col'
        call trim_c(f_col, nc_f_col)

        write(*,*)
        write(*,'(a)') ' Writing output file: '//f_col(1:nc_f_col)

        call get_lun(nu_col)
        open(nu_col,f_col(1:nc_f_col), status='unknown')
      
        if (write_headers(1:1) == 'Y') then
          do i = 1, 11
            write(nu_col, '(a)') char_head(i)
          end do

          write(nu_col, '(8I10)') (int_head(i), i=1, 48)

          write(nu_col, '(5e15.7)') (real_head(i), i = 1, 50)

          do i = 1, int_head(16)
            write(nu_col,'(a)') comments(i)
          end do
        end if

        

        write(nu_col,'(11x,a, 11x,a)') 'time', 'gmim'
      
        do i=1, npy
          write(nu_col,'(1x,es14.7, 1x, es14.7)') real(i-1)/sps, y(i)
        end do

        deallocate(y)

        close(unit=nu_col)

      END DO loop_over_files

      close(nu_ctl)

      stop
      end
! ------------------ END SMC2COL -----------------------------

      include '\forprogs\SMCRead.for'
      include '\forprogs\SMCWrite.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\upstr.for'
      include '\forprogs\distaz.for'


