
* ---------------------------------------------------------------- SMC_Add
       Program SMC_Add

* Reads station name from control file and writes it in text header(6)(11:40),
* obtains orientation from text header(6)(53:57) and writes the appropriate
* number in the integer headers 13, 14.

* For flexibility, information is read from a control file.
* Here is an example:
*Control file for program SMC_Add
*Path for input SMC files:
* \psv\pet_ms\usgs\
*Path for output SMC files (can be same as input, then files are overwritten)
* \psv\pet_ms\usgs\
*Name of SMC file, followed by station name:
*319A.SMC
* Butler Valley Sta. 2
*319B.SMC
* Butler Valley Sta. 2
*319C.SMC
* Butler Valley Sta. 2
*334A.SMC
* Ferndale FS
*334B.SMC
* Ferndale FS
*334C.SMC
* Ferndale FS
*stop

*  Dates:  02/28/01 - Written by D. Boore (based on SMCAddSN.FOR)
*        07/17/03 - Add sps to smc_npts parameter list
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats

      character precision*8
      

      real  rhead(50)

      integer ihead(48)

      real y(:)
      allocatable :: y

      character chead(11)*80, comments(200)*80, staname*30, orient*5
      character path_in*50, path_out*50
      character f_ctl*40, f_in*70, f_out*70, f_smc*40, f_sum*40
      character buf*80, buf_stop*4, ifrmt_c*6, date_c*8, time_c*11

      logical f_ctl_exist

* Open Input file:

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') stop
        inquire(file=f_ctl, exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl, status='unknown')

      call skip(nu_ctl,2)
      path_in = ' '
      read(nu_ctl,'(a)') path_in
      call trim_c(path_in,nc_path_in)

      call skip(nu_ctl,1)
      path_out = ' '
      read(nu_ctl,'(a)') path_out
      call trim_c(path_out,nc_path_out)

      call skip(nu_ctl,1)

1000  continue

      f_smc = ' '
      read(nu_ctl,'(a)') f_smc
      call trim_c(f_smc,nc_f_smc)
      buf_stop = ' '
      buf_stop = f_smc(1:4)
      call upstr(buf_stop)

      if(buf_stop .eq. 'STOP') then
        close(nu_ctl)
        stop
      end if

      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf,nc_buf)
      staname = ' '
      staname = buf

      f_in = ' '
      f_in = path_in(1:nc_path_in)//f_smc(1:nc_f_smc)

      f_out = ' '
      f_out = path_out(1:nc_path_out)//f_smc(1:nc_f_smc)

* DEBUG
      write(*,'(2a)') ' f_in = ', f_in
      write(*,'(2a)') ' f_out = ', f_out
* DEBUG

      call smc_npts(f_in,npts_smc, sps)
      allocate(y(npts_smc))

      tskip = 0.0
      tlength = 9999.9
      call SMCRead(f_in, tskip, tlength,
     :                   y, chead, ihead, rhead, comments)

      inull = ihead(1)
      rnull = rhead(1)

      chead(6)(11:40) = ' '
      chead(6)(11:40) = staname

      ihead(13) = inull
      ihead(14) = inull

      orient = ' '
      orient = chead(6)(53:57)
      
      call trim_c(orient,nc_orient)
      call upstr(orient)

      if (nc_orient .gt. 0) then
        if(orient(1:2) .eq. 'UP') then
          ihead(13) = 0
        else
          if(nc_orient .eq. 1) ifrmt_c = '(i1)'        
          if(nc_orient .eq. 2) ifrmt_c = '(i2)'        
          if(nc_orient .eq. 3) ifrmt_c = '(i3)'        
          if(nc_orient .eq. 4) ifrmt_c = '(i4)'        
          if(nc_orient .eq. 5) ifrmt_c = '(i5)'       
          if(nc_orient .eq. 6) ifrmt_c = '(i6)'   ! out of luck if ichars > 6!
          read(orient(1:nc_orient),ifrmt_c) ihead(14)
        end if
      end if

      iflag = 0 ! not used by current PC version of datetime; Chris
                ! Stephens would modify datetime.for to include Unix routines
      call datetime(date_c, time_c, iflag)
      ncomments = ihead(16)
      ncomments = ncomments + 1
      comments(ncomments) = '|'
      ncomments = ncomments + 1
      comments(ncomments) = '| Using program SMC_Add:'
      ncomments = ncomments + 1
      comments(ncomments) = 
     :  '|   Station name added to text header on '//date_c//
     :  ' at time '//time_c//':'
      if (nc_orient .ne. 0) then
        ncomments = ncomments + 1
        comments(ncomments) = 
     :  '|   and orientation placed in integer headers'
      end if
      ncomments = ncomments + 1
      comments(ncomments) = '|'
      ihead(16) = ncomments

      precision = ' '
      precision = 'STANDARD'
      ihead(47) = ihead(1)

      itype = -1
      call SMCWrite(f_out, precision,
     :              y, chead, itype, 
     :                 ihead, rhead, comments)

* That should be it; deallocate the y array and loop back for another file
 
      deallocate(y)

      goto 1000

      end
* ---------------------------------------------------------------- SMC_Add


      include '\forprogs\skip.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmax.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\datetime.for'

                     