
* ---------------------------------------------------------------- smc_pad
       Program smc_pad

* Add leading zeros to smc file and write as a new file
* The program skips through the input time series to the first zero crossing.

* For flexibility, information is read from a control file.
* Here is an example:
*!Control file for program SMC_Pad
*! Replace discarded portions of original data with zeros (Y/N)?  By doing this,
*! two horizontal components with the same number of time points before
*! padding (the usual case) will have the same number after padding and can
*! be rotated.
* Y
*!Length of leading pad in seconds (BAP suggests 1.5*nroll/fc):
* 9.0
*!Go to first zero crossing?
* Y
*!Use first point if smaller than first zero crossing?
* Y
*!Length of trailing pad in seconds (BAP suggests 1.5*nroll/fc):
* 9.0
*!Go to last zero crossing?
* Y
*!Use last point if smaller than last zero crossing?
* Y
*!        11/03/06 - If zcross_l or zcross_t = F and use_first or use_last = T, 
*!                   pad with values equal to the last points.  Allow this option
*!                   to deal with cases such as filtering a displacement
*!                   time series with a low-frequency drift, which will have
*!                   a step offset if pad with zeros at the end.  Be careful in
*!                   in using this option for the first points, however (in
*!                   a future version I may eliminate the option for the beginning,
*!                   as generally the leading pad should be zero).  The need for
*!                   this option arose when I tried to filter a displacement
*!                   obtained by integrating a zeroth-order corrected acceleration.
*!                   The results were different than filtering the acceleration
*!                   before integration.
*!                   IF WANT TO PAD WITH ZEROS, STARTING AT 
*!                   BEGINNING AND END OF DATA, SET ZCROSS_L, ZCROSS_T = F AND
*!                   USE_FIRST, USE_LAST = F (This is potentially confusing, and
*!                   I may remove the option of padding with the first and last
*!                   values in a later version of the program)
*!
*! Remove mean before determining segment to be padded?
*! If Yes, the mean will be determined from the whole record.  If this
*! is not desireable, remove the mean outside of this program
* N
*! Remove mean after determining segment to be padded (the segment
*! will probably be different than the segment originally used to determine the
*! mean, and therefore the mean will no longer be 0.0. But then after removing
*! this mean, the segment to be padded will not be at 0.0 values, but I hope this
*! is a small problem.  I probably should iterate):
* Y
*!Time for taper at beginning of time series, before adding zeros
*  0.0
*!Time for taper at end of time series, before adding zeros
*  0.0
*!character string to append to filename:
* .pad
*!Name of SMC file (output name = stem.pad):
* 319A.SMC
* 319B.SMC
* 319C.SMC
* 334A.SMC
* 334B.SMC
* 334C.SMC
*stop

* Dates: 04/15/02 - Written by D. Boore (based on SMC_Y1YM.FOR, and borrowing
*                   code from Converse's BAPPAD.FOR).
*        04/16/02 - Add option to include trailing zeros (needed for acausal
*                   filtering).
*        08/13/02 - Added zcross to output comments, and added option to
*                   use (or not use, the reason I put in this option), the
*                   first or last points if zcross = T and they are smaller
*                   than the first or last zero crossings, respectively.
*                   Also, added  "abs" function for peak motion in definition
*                   smidge.  Also made "small" much smaller.
*        10/21/02 - Added stuff to comment headers
*        12/16/02 - Add options for removing means before and after segment
*                   to be padded has been determined.
*        12/18/02 - Changed name of get_mean to get_avg and added argument 
*                   icorr_cumavg to call to get_avg
*        05/31/03 - Corrected comments written to file (they incorrectly 
*                   stated that the mean as removed before adding zeros;
*                   I now allow for both possibilities).
*        11/03/03 - Added write statement to file comments indicating if 
*                    means have NOT been removed.
*        11/26/03 - Put padding into a subroutine (so can use it in bl_fltr)
*        06/04/04 - Replaced tlz, ttz with nlz, ntz.  Also, allow the 
*                   discarded data to be replaced with zeros (thus 
*                   preserving the length of the input time series).
*        03/13/06 - Remove f_smc from smcpadf
*        11/03/06 - Modified smcpadf to trick it into using the first or last
*                   values of the unpadded time series for the pad values
*                   (see comments in the sample control file above for
*                   the reason for doing so).
*        01/24/07 - Added option to taper end of time series before adding trailing
*                   zero pad.
*        01/25/07 - Ask for file extension to add to name of output file.
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats
*        03/24/08 - Include a taper at the beginning pad
*                   Use smc_npts rather than smc_nsps
  

      character precision*8



      real  rhead(50)

      integer ihead(48)

      real y(:)
      allocatable :: y

      character chead(11)*80, comments(800)*80
      character f_ctl*100, f_out*120, f_smc*120
      character buf_stop*4, buf*10
      character ctl_cmmnts(200)*80
      character string_to_append*40

      character date_c*8, time_c*11

      logical f_ctl_exist, zcross_l, zcross_t, use_first, use_last
      logical remove_mean_before, remove_mean_after,
     :  replace_discarded_data_with_zeros

* Open Input file:

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = smc_pad.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'smc_pad.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl(1:nc_f_ctl), status='unknown')

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) .eq. 'Y') then
        replace_discarded_data_with_zeros = .true.
      else
        replace_discarded_data_with_zeros = .false.
      end if

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      read(nu_ctl,*) tlz

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) .eq. 'Y') then
        zcross_l = .true.
      else
        zcross_l = .false.
      end if

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) .eq. 'Y') then
        use_first = .true.
      else
        use_first = .false.
      end if

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      read(nu_ctl,*) ttz

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) .eq. 'Y') then
        zcross_t = .true.
      else
        zcross_t = .false.
      end if

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) .eq. 'Y') then
        use_last = .true.
      else
        use_last = .false.
      end if

      write(*,'(a)') ' tlz, ttz = '
      write(*, *)      tlz, ttz

      write(*,'(a)') ' zcross_l, zcross_t = '
      write(*, *)      zcross_l, zcross_t

      write(*,'(a)') ' use_first, use_last = '
      write(*, *)      use_first, use_last


      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) .eq. 'Y') then
        remove_mean_before = .true.
      else
        remove_mean_before = .false.
      end if
      write(*,'(a,1x,l1)') 
     :      ' remove_mean_before = ', remove_mean_before

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) .eq. 'Y') then
        remove_mean_after = .true.
      else
        remove_mean_after = .false.
      end if
      write(*,'(a,1x,l1)') 
     :      ' remove_mean_after = ', remove_mean_after

* OPTION TO TAPER BEGINNING AND END OF TIME SERIES BEFORE ADDING TRAILING ZERO PAD

      call skipcmnt(nu_ctl,ctl_cmmnts, nctl_cmmnts)
      read(nu_ctl,*) taper_beginning
  
      call skipcmnt(nu_ctl,ctl_cmmnts, nctl_cmmnts)
      read(nu_ctl,*) taper_end
  
* GET STRING TO APPEND TO FILE NAME:

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      string_to_append = ' '
      read(nu_ctl,'(a)') string_to_append
      call trim_c(string_to_append, nc_s2appnd)
 
      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)

1000  continue

      f_smc = ' '
      read(nu_ctl,'(a)') f_smc
      call trim_c(f_smc,nc_f_smc)
      buf_stop = ' '
      buf_stop = f_smc(1:4)
      call upstr(buf_stop)

      if(buf_stop .eq. 'STOP') then
        close(nu_ctl)
        stop
      end if

      write(*,'(a)') ' f_smc = '//f_smc(1:nc_f_smc)

* DEBUG
      print *,' f_smc = '//f_smc(1:nc_f_smc)
* DEBUG
      
      call smc_npts(f_smc(1:nc_f_smc), npts_smc, sps)

      nlz = tlz * sps
      npts_plus_lz = npts_smc + nlz

      ntz = ttz * sps
      npts_plus_lz_tz = npts_smc + nlz + ntz

      npts4alloc = npts_plus_lz_tz
      allocate(y(npts4alloc))

      tskip = 0.0
      tlength = -1
      call SMCRead(f_smc(1:nc_f_smc), tskip, tlength,
     :                   y, chead, ihead, rhead, comments)

      precision = ' '
      if (ihead(47) .eq. 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
      npts_in = npts_smc
      ncomments = ihead(16)
      ncomments = ncomments + 1
      comments(ncomments) = '|'
      ncomments = ncomments + 1
      write(comments(ncomments),'(a)') 
     :     '| To input file '//f_smc(1:nc_f_smc) 
      ncomments = ncomments + 1
      write(comments(ncomments),'(a)') 
     :     '| program SMC_PAD did the following:'

      print *,' npts_in, sps, nlz, ntz, 
     :  replace_discarded_data_with_zeros,
     :  taper_end,   
     :  remove_mean_before, remove_mean_after, 
     :  zcross_l, use_first, zcross_t, use_last'

      print *, npts_in, sps, nlz, ntz, 
     :  replace_discarded_data_with_zeros,
     :  taper_end,
     :  remove_mean_before, remove_mean_after, 
     :  zcross_l, use_first, zcross_t, use_last

      call smcpadf(y, npts_in, sps, nlz, ntz, 
     :  replace_discarded_data_with_zeros,
     :  taper_beginning, taper_end, 
     :  npts_out, izcl, izct,  
     :  remove_mean_before, remove_mean_after, 
     :  zcross_l, use_first, zcross_t, use_last,
     :  comments, ncomments)    

 
* Store new ncomments, npts:

      ihead(16) = ncomments            
      ihead(17) = npts_out

* Construct file name for output
      f_out = ' '
      f_out = f_smc(1:nc_f_smc)//string_to_append(1:nc_s2appnd)

      call trim_c(f_out, nc_f_out)
      write(*,'(a)') ' f_out = '//f_out(1:nc_f_out)

      ncomments = ncomments + 1
      write(comments(ncomments),'(a)') 
     :     '| resulting in output file '//f_out(1:nc_f_out)
      iflag = 0 ! not used by current PC version of datetime; Chris
                ! Stephens would modify datetime.for to include Unix routines
      call datetime(date_c, time_c, iflag)
      ncomments = ncomments + 1
      comments(ncomments) = 
     :     '| on '//date_c//' at time '//time_c

      ncomments = ncomments + 1
      comments(ncomments) = '|'

* Write the output file:

      itype = -1
      call SMCWrite(f_out, precision,
     :              y, chead, itype, 
     :                    ihead, rhead, comments)

* That should be it; deallocate the y array and loop back for another file
 
      deallocate(y)

      goto 1000

      end
* ---------------------------------------------------------------- smc_pad


      include '\forprogs\smcpadf.for'
      include '\forprogs\skip.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmax.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\datetime.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_avg.for'
      include '\forprogs\distaz.for'

                     