
* ---------------------------------------------------------------- smc_y1ym
       Program smc_y1ym

* Get initial value (y(1)) and peak value of time series

* For flexibility, information is read from a control file.
* Here is an example:
*!Control file for program SMC_Y1Ym
*!Name of output file:
* smc_y1ym.out
*!Name of SMC file:
* 319A.SMC
* 319B.SMC
* 319C.SMC
* 334A.SMC
* 334B.SMC
* 334C.SMC
*stop

*  Dates:  04/08/02 - Written by D. Boore (based on SMC_Long.FOR)
*          04/11/02 - Sort the arrays in decreasing order when printed
*        07/17/03 - Add sps to smc_npts parameter list
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats

      character precision*8


      real  rhead(50), y1(1000), ym(1000), yrat(1000)

      integer ihead(48), indx(1000), n4out(1000)

      real y(:)
      allocatable :: y

      character chead(11)*80, comments(200)*80
      character f_ctl*40, f_out*70, f_smc(1000)*40
      character buf_stop*4
      character ctl_cmmnts(10)*80

      logical f_ctl_exist

* Open Input file:

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = smc_y1ym.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'smc_y1ym.ctl'
        inquire(file=f_ctl, exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl, status='unknown')


      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      f_out = ' '
      read(nu_ctl,'(a)') f_out
      call trim_c(f_out, nc_f_out)
      call get_lun(nu_out)
      open(unit=nu_out, file=f_out(1:nc_f_out), status='unknown')
      write(*,'(a)') ' f_out = '//f_out(1:nc_f_out)
      write(nu_out,'(26x,a, 9x,a, 9x,a, 1x,a)')
     :              'f_smc', 'y1', 'ym', '100*y1/abs(ym)'
 
      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)

      nf = 0

1000  continue

      f_smc(nf+1) = ' '
      read(nu_ctl,'(a)') f_smc(nf+1)
      call trim_c(f_smc(nf+1),nc_f_smc)
      buf_stop = ' '
      buf_stop = f_smc(nf+1)(1:4)
      call upstr(buf_stop)

      if(buf_stop .eq. 'STOP') then
        close(nu_ctl)
        go to 2000
      end if

      nf = nf + 1

      write(*,'(a)') ' f_smc = '//f_smc(nf)(1:nc_f_smc)

      call smc_npts(f_smc(nf),npts_smc,sps)
      allocate(y(npts_smc))

      tskip = 0.0
      tlength = 9999.9
      call SMCRead(f_smc(nf), tskip, tlength,
     :                   y, chead, ihead, rhead, comments)

      precision = ' '
      if (ihead(47) .eq. 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
      y1(nf) = y(1)

      call mnmaxidx(y, 1, ihead(17), 1,
     :             ymin, ymax, indx_min, indx_max) 
      ym(nf) = ymax
      if (abs(ymin) .gt. abs(ymax)) ym(nf) = ymin

      yrat(nf) = 100.0*abs(y1(nf)/ym(nf))

      n4out(nf) = min0(30, nc_f_smc)
      
* That should be it; deallocate the y array and loop back for another file
 
      deallocate(y)

      goto 1000

2000  continue

* Now sort:
      call indexx(nf, yrat, indx)
      
      do i = nf, 1, -1
        j = indx(i)
        write(nu_out,'(1x,a30, 1p, 2(1x,e10.3), 5x,e10.3)')
     :    f_smc(j)(1:n4out(j)), y1(j), ym(j), yrat(j)
      end do

      close(nu_out)

      end
* ---------------------------------------------------------------- smc_long


      include '\forprogs\skip.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmax.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\datetime.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\indexx.for'

                     