      Program SMCOffSt

* Read in a smc file, apply an offset, and write as a new smc file.
* The intent is to mimic a digitizer that does not have a perfect 0.0

* Input from a control file. Here is an example:
*! Control file for program SMCOffSt
*!Place in which to overwrite existing character with stem that follows
*!after period (-1 means just to the left; 
*!0 will insert to the left of the period unless the stem is 7 or 8 characters,
*!in which case it will overwrite the 7th & 8th characters):
* -5 
*!String to write into input name:
* o1
*!
*! a_offset_min, a_offset_max (used to introduce an offset on the trace, chosen
*! randomly assuming a uniform distribution between a_offset_min
*! and a_offset_max); units of cm/s/s.
*  -6.0 +6.0
*! Seed for random number generator (a negative integer)
* -309
*! Names of input files
*smc1.smc
*smc2.smc
*Stop

* Dates: 10/08/02 - Written by D. Boore, based on smc_dig.for
*        10/22/02 - Stuff added to comment headers
*        07/17/03 - Add sps to smc_npts parameter list
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats

      character precision*8

 
      character char_head(11)*80, comments(500)*80, cmnts2skip(50)*80
      integer int_head(48)
      real real_head(50)
      real y_in(:), y_out(:)
      allocatable :: y_in, y_out
      character buf*80, buf4*4, f_in*40, f_out*40, astring*4

      character date_c*8, time_c*11

      character f_ctl*40
      logical f_ctl_exist, f_in_exist

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=smcoffst.ctl;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smcoffst.ctl'
        inquire(file=f_ctl, exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(nu_ctl,f_ctl,status='unknown')


      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl, *) n4name
      write(*,'(a,1x,i3)') ' n4name = ', n4name

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      astring = ' '
      read(nu_ctl,'(a)') astring
      call trim_c(astring, nc_astring)


      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) a_offset_min, a_offset_max

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) idum

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)

      nfiles = 0

1000  continue

        buf = ' '
        read(nu_ctl,'(a)') buf
        call trim_c(buf, nc_buf)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        if (buf4 .eq. 'STOP') then
          close(nu_ctl)
          stop
        end if

        f_in = ' '
        f_in = buf(1:nc_buf)
        call trim_c(f_in,nc_f_in)
        inquire(file=f_in(1:nc_f_in), exist=f_in_exist)
        if (.not. f_in_exist) then
            write(*,'(a)') ' ****** FILE '//f_in(1:nc_f_in)// 
     :            ' DOES NOT EXIST; SKIPPING!!! ******* '
            go to 1000
        end if

* Passed tests; proceed

        write(*,*)
        write(*,'(1x,a)') 
     :     'Processing file:', f_in(1:nc_f_in)

        nfiles = nfiles + 1

        call smc_npts(f_in(1:nc_f_in), npts, sps)

        write(*,'(a,1x,i8)') 
     :    ' For file '//f_in(1:nc_f_in)//', npts = ', npts

        allocate(y_in(npts), y_out(npts))

        tskip = 0.0
        tlength = -1.0
        call SMCRead(f_in(1:nc_f_in), tskip, tlength,
     :             y_in, char_head, int_head, real_head, comments)

      precision = ' '
      if (int_head(47) .eq. 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
        ny = int_head(17)
        write(*,'(a,1x,i8)') ' For file '//f_in(1:nc_f_in)//
     :                     ', ny = ', ny

        a_offset = a_offset_min + ran1(idum) * 
     :             (a_offset_max - a_offset_min)

        write(*,'(a,1x,1p,e12.5)') ' a_offset = ', a_offset

* Apply offset:
        do i = 1, ny
          y_out(i) = y_in(i) + a_offset
        end do

* Write the new smc file:

* Construct output filename:

        indp = index(f_in,'.')
        f_out = ' '
        f_out = f_in(1:nc_f_in)
        f_out(indp+n4name:indp+n4name+nc_astring-1) = 
     :        astring(1:nc_astring)
        
        call trim_c(f_out, nc_f_out)

        write(*,'(1x,a)') 
     :     'Writing output to file:', f_out(1:nc_f_out)

        ncomments = int_head(16)
        ncomments = ncomments + 1
        comments(ncomments) = '|'
        ncomments = ncomments + 1
        comments(ncomments) = '|  Input file '//f_in(1:nc_f_in)
        ncomments = ncomments + 1
        comments(ncomments) = '|  Offset added using program SMCOffSt,'
        ncomments = ncomments + 1
        comments(ncomments) = '|    with a_offset = '
        write(comments(ncomments)(22:33),'(1p,e12.5)') a_offset
        iflag = 0
        call datetime(date_c, time_c, iflag)
        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '|   file '//f_out(1:nc_f_out)//' written on '//
     :       date_c//' at time '//time_c
        ncomments = ncomments + 1
        comments(ncomments) = '|'
        int_head(16) = ncomments

        itype = -1  ! use info from header
        call SMCWrite(f_out(1:nc_f_out), precision,
     :                y_out, char_head, itype, 
     :                      int_head, real_head, comments)

        deallocate (y_in, y_out)

      go to 1000
      end

      include '\forprogs\get_lun.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smc_nsps.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\datetime.for'
      include '\forprogs\ran1.for'

 

            