* ------------------------------------------------------------------- smcrmvsn
      Program smcrmvsn


* Remove a single cycle of a sine from acceleration.  This is to
* study Norm Abrahamson's "fling-step".  

* Input from a control file; here is an example:
*!Control file for program SMCRmvSn
*! As many comment lines as desired, each starting with "!"
*! The string "pp:" indicates a new set of processing parameters
*! to be applied to the following smc files.  The parameters are given on the
*! following 4 lines.
*!Name of summary file:
* smcrmvsn.sum
*!Files and processing parameters follow.  The first file is the input file,
*! the second file is the output file.
*pp:
*    30.0 46.5  610.0 ! T1, T2, D
* \psv\taiwan99\cc_bl\68nt2v0a.smc
* rmvsn_01.smc
*pp:
*    30.0 44.0  610.0 ! T1, T2, D
* \psv\taiwan99\cc_bl\68nt2v0a.smc
* rmvsn_02.smc
*stop

* The sin pulse goes from T1 to T2, and has an amplitude 2*pi*D/(T2-T1)**2,
* where D is the height of the displacement level.  Note that the sign of D
* is important.  Also note that the double integration of the input file, with
* no mean removed, should give a more-or-less flat residual displacement.  One 
* way of determining the parameter D is to use smc2asc to make a file with the
* acc, vel, and dis time series, then in CoPlot fit the mean to the later 
* portion of displacement.

* Dates: 02/10/03 - Written by D.M. Boore, patterned after SMC2FS2
*        07/17/03 - Add sps to smc_npts parameter list
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats

      character precision*8


      real ts_in(:), ts_out(:)
      allocatable :: ts_in, ts_out

      integer npts, nc_f_smc

      character char_head(11)*80, comments(500)*80, cmnts2skip(50)*80
      real real_head(50)
      integer int_head(48)

      character buf*80, buf_upper*4, 
     :        f_ctl*70, f_in*70, f_out*70, f_sum*40

      character date_c*8, time_c*11

      logical f_ctl_exist, f_in_exist

      pi = 4.0*atan(1.0)
      twopi = 2.0 * pi

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=smcrmvsn.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smcrmvsn.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') ' Output of program smcrmvsn:'
      write(nu_sum,'(2a)') '   Control file = ', f_ctl

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

1000  continue
      buf = ' '
      read(nu_ctl,'(a)',end = 9999) buf
      call trim_c(buf,nc_buf)
      buf_upper = ' '
      buf_upper = buf(1:4)
      call upstr(buf_upper)
      if(nc_buf .eq. 0 .or. buf(1:1) .eq. '!') go to 1000
      if(buf_upper(1:4) .eq. 'STOP') go to 9999
      
      if(buf_upper(1:3) .eq. 'PP:') then   ! a new set of processing parameters
        read(nu_ctl,*) T1, T2, D
        A = twopi * D/(T2-T1)**2
        write(nu_sum,'(a)') 
     :     ' T1, T2, D, A ='
        write(nu_sum,'(4(1x,f8.3))') 
     :       T1, T2, D, A

        go to 1000
      end if

      f_in = ' '
      f_in = buf(1:nc_buf)
      nc_f_in = nc_buf     
      inquire(file=f_in(1:nc_f_in), exist=f_in_exist)
      if (.not. f_in_exist) then
          write(*,'(a)') ' ****** FILE '//f_in(1:nc_f_in)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          go to 1000
      end if

      f_out = ' '
      read(nu_ctl,'(a)',end = 9999) f_out
      call trim_c(f_out, nc_f_out)

* Passed tests; proceed

* Get number of points on this loop:

      call smc_npts(f_in(1:nc_f_in), npts, sps)

      write(nu_sum,*)
      write(nu_sum, '(a, 2(1x,i5))') ' npts = ', npts

      allocate( ts_in(npts), ts_out(npts) )

* Now read the data from the files:

      write(*,*)
      write(*,'(1x,a)') 
     :   'Processing file:', f_in(1:nc_f_in)

      tskip = 0.0
      tlength = -1.0      
      call SMCRead(f_in(1:nc_f_in), tskip, tlength,
     :               ts_in,  
     :               char_head, int_head, real_head, comments)
      precision = ' '
      if (int_head(47) .eq. 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
      sps = real_head(2)
      npts_smc = int_head(17)
      dt = 1.0/sps
      t_smc = float(npts_smc-1)*dt
      nt1 = T1 * sps + 1
      nt2 = T2 * sps + 1

      do i = 1, npts_smc
        ts_out(i) = ts_in(i)
      end do

      do i = nt1, nt2
        ts_out(i) = ts_in(i) - 
     :        A*sin(twopi*float(i-nt1)/float(nt2-nt1))
      end do

      deallocate( ts_in )

* Write output:

      ncomments = int_head(16)
      ncomments = ncomments + 1
      comments(ncomments) = '|'
      ncomments = ncomments + 1
      write(comments(ncomments),'(a)') 
     :     '| From input file '//f_in(1:nc_f_in)//
     :     ', program SMCRmvSn removed a sinusoid '
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,f8.3,a,f8.3,a,f8.3,a)') 
     :     '| of amplitude A = ', A, 
     :     ' from T1 = ', T1, ' to T2 = ', T2, 'sec'
      iflag = 0 ! not used by current PC version of datetime; Chris
                ! Stephens would modify datetime.for to include Unix routines
      call datetime(date_c, time_c, iflag)
      ncomments = ncomments + 1
      comments(ncomments) = 
     :     '| on '//date_c//' at time '//time_c
      ncomments = ncomments + 1
      write(comments(ncomments),'(a)') 
     :     '| resulting in output file '//f_out(1:nc_f_out)

      ncomments = ncomments + 1
      comments(ncomments) = '|'
      int_head(16) = ncomments            

      itype = -1
      call SMCWrite(f_out(1:nc_f_out), precision,
     :               ts_out, char_head, itype, 
     :                    int_head, real_head, comments)

      deallocate (ts_out)
      go to 1000

9999  continue

* Close control and output files:

      close(nu_ctl)
      close(nu_sum)

      stop
      end
* ------------------------------------------------------------------- smcrmvsn


      include '\forprogs\fork.for'
*      include '\forprogs\forkdp.for'
      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\imnmax.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\datetime.for'

